@@ -4,7 +4,7 @@ interface to report text information about images.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -22,7 +22,7 @@ class ImageAccInfoAdapter(DefaultAccInfoAdapter):
   Overrides L{DefaultNavAdapter} to provide text information about images. 
   Expects the subject to be a L{POR}.
 
-  Adapts accessibles that provide the L{Interfaces.IImage} interface and
+  Adapts accessibles that provide the Image interface and
   have a role of image or icon.
   '''
   provides = [IAccessibleInfo]
@@ -56,4 +56,4 @@ class ImageAccInfoAdapter(DefaultAccInfoAdapter):
     '''
     acc = self.accessible
     s = acc.name or Interfaces.IImage(acc).imageDescription
-    return unicode(s, 'utf-8')
\ No newline at end of file
+    return unicode(s, 'utf-8')
