@@ -3,28 +3,28 @@ Defines a L{Task} to execute when a text or numeric property changes.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import EventTask, Constants
+import Base, EventTask, AEConstants
 import AEEvent
 
 class PropertyTask(EventTask.EventTask):
   '''
   Executed when a text or numeric property change occurs. 
 
-  This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor} using the L{Constants.registerTaskType} function
-  when this module is first imported. The L{AEMonitor} can use this
-  information to build its menus.
+  This class registers its name and whether it should be monitored by default
+  in an L{AEMonitor} using the L{Base.registerTaskType} function when this
+  module is first imported. The L{AEMonitor} can use this information to build
+  its menus.
   '''
-  Constants.registerTaskType('PropertyTask', False)
+  Base.registerTaskType('PropertyTask', False)
 
   def getEventType(self):
     '''
@@ -44,8 +44,7 @@ class PropertyTask(EventTask.EventTask):
     @type name: string
     @param value: New value of the property
     @type value: string or number
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
     pass
@@ -61,8 +60,7 @@ class PropertyTask(EventTask.EventTask):
     @type name: string
     @param value: New value of the property
     @type value: string or number
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
